#ifndef FrameCPP_VERSION_8_FrTOCAdcData_HH
#define FrameCPP_VERSION_8_FrTOCAdcData_HH

#include "framecpp/Common/FrTOC.hh"
#include "framecpp/Version8/STRING.hh"

namespace FrameCPP
{
  namespace Common
  {
    class TOCInfo;
  }

  namespace Version_8
  {
    //===================================================================
    /// \brief Indexed elements of the FrAdcData structure.
    //===================================================================
    class FrTOCAdcData
    {
    public:
      typedef INT_4U	nadc_type;
      typedef STRING	name_type;
      typedef INT_4U	channel_id_type;
      typedef INT_4U	group_id_type;
      typedef INT_8U	position_type;

      struct adc_info_type {
	//---------------------------------------------------------------
	/// \brief Channel ID of ADC
	//---------------------------------------------------------------
	channel_id_type			m_channelID;
	//---------------------------------------------------------------
	/// \brief Group ID of ADC
	//---------------------------------------------------------------
	group_id_type			m_groupID;
	//---------------------------------------------------------------
	/// \brief FrAdcData offset positions.
	///
	/// FrAdcData offset positions, in bytes,
	/// from the beginning of the file.
	/// The size of the array is nFrame
	//---------------------------------------------------------------
	std::vector< position_type >	m_positionADC;
      };

      //-----------------------------------------------------------------
      /// \brief Container of indexed FrAdcData elements.
      ///
      /// This structure is ordered by the name of the FrAdcData
      /// channel name.
      /// There is one element for each channel of indexed data.
      //-----------------------------------------------------------------
      typedef std::map< name_type, adc_info_type >	MapADC_type;

      //-----------------------------------------------------------------
      /// \brief Default constructor.
      //-----------------------------------------------------------------
      FrTOCAdcData( );

      //-----------------------------------------------------------------
      /// \brief Number of bytes needed to write this structure
      ///
      /// \param[in] Stream
      ///     The stream from which to the object is being read or
      ///     written.
      ///
      /// \return
      ///     The number of bytes need to read or write this object.
      //-----------------------------------------------------------------
      Common::FrameSpec::size_type
      Bytes( const Common::StreamBase& Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Return all channels
      ///
      /// \return
      ///     Constant container to the indexed FrAdcData elements.
      //-----------------------------------------------------------------
      const MapADC_type& GetADC( ) const;

      //-----------------------------------------------------------------
      /// \brief Return the indexed data for a named channel.
      ///
      /// \param[in] Channel
      ///     The name of the channel.
      ///
      /// \return
      ///     Constant iterator to the indexed FrAdcData element.
      //-----------------------------------------------------------------
      MapADC_type::const_iterator GetADC( const std::string& Name ) const;

      //-----------------------------------------------------------------
      /// \brief Return the indexed data for an indexed channel.
      ///
      /// \param[in] Channel
      ///     The numeric offset of the channel.
      ///
      /// \return
      ///     Constant iterator to the indexed FrAdcData element.
      //-----------------------------------------------------------------
      MapADC_type::const_iterator GetADC( INT_4U index ) const;

      //-----------------------------------------------------------------
      /// \brief Gather TOC info for FrAdcData being written.
      ///
      /// \param[in] Info
      ///     Information
      ///
      /// \param[in] FrameOffset
      ///     The frame offset of the frame being written.
      ///     The frame offsets start at 0 (zero).
      ///
      /// \param[in] Position
      ///     The byte offset from the start of the file
      ///     where the structure is written.
      //-----------------------------------------------------------------
      void QueryAdc( const Common::TOCInfo& Info,
		     INT_4U FrameOffset,
		     INT_8U Position );

      //-----------------------------------------------------------------
      /// \brief equality operator
      ///
      /// \param[in] RHS
      ///     The FrTOCAdcData object to be compared.
      ///
      /// \return
      ///     The value true is returned if this object is equivelent
      ///     to the RHS instance; false otherwise.
      //-----------------------------------------------------------------
      bool operator==( const FrTOCAdcData& RHS ) const;

      //-----------------------------------------------------------------
      /// \brief The description of structure
      ///
      /// \return
      ///     A Description object which describes this structure as
      ///     specified by the frame specification.
      //-----------------------------------------------------------------
      template< typename SE >
      static void Description( Common::Description& Desc );

    protected:
      typedef std::vector< name_type >	key_container_type;
      key_container_type	m_keys;
      MapADC_type		m_info;

      //-----------------------------------------------------------------
      /// \brief Stream constructor
      ///
      /// \param[in] Stream
      ///     The input stream from where the object is being read.
      ///
      /// \param[in] FrameCount
      ///     The frame offset.
      //-----------------------------------------------------------------
      FrTOCAdcData( Common::IStream& Stream, INT_4U FrameCount );

      //-----------------------------------------------------------------
      /// \brief Iterate over contents.
      ///
      /// \param[in] Info
      ///     Specifies the type of information to be searched.
      ///
      /// \param[in] Action
      ///     Action to be taken for each piece of information found.
      //-----------------------------------------------------------------
      void forEach( Common::FrTOC::query_info_type Info,
		    Common::FrTOC::FunctionBase& Action ) const;

      //-----------------------------------------------------------------
      /// \brief Write the structure to the stream
      ///
      /// \param[in] Stream
      ///     The output stream where the object is to be written.
      //-----------------------------------------------------------------
      void write( Common::OStream& Stream ) const;
    };

    inline Common::FrameSpec::size_type FrTOCAdcData::
    Bytes( const Common::StreamBase& Stream ) const
    {
      Common::FrameSpec::size_type 
	retval = sizeof( nadc_type );
      if ( m_info.size( ) )
      {
	retval
	  += ( m_info.size( )
	       * ( sizeof( channel_id_type )
		   + sizeof( group_id_type )
		   + ( sizeof( position_type )
		       * m_info.begin( )->second.m_positionADC.size( ) )
		   ) );
      }
      for ( MapADC_type::const_iterator
	      cur = m_info.begin( ),
	      last = m_info.end( );
	    cur != last;
	    ++cur )
      {
	retval += cur->first.Bytes( );
      }
      return retval;
    }

    template< typename SE >
    void FrTOCAdcData::
    Description( Common::Description& Desc )
    {
      Desc( SE( "nADC", "INT_4U",
		"Number of unique FrAdcData names in file." ) );
      Desc( SE( "name", "STRING[nADC]", "Array of FrAdcData names" ) );
      Desc( SE( "channelID", "INT_4U[nADC]",
		"Array of ADC channel IDs" ) );
      Desc( SE( "groupID", "INT_4U[nADC]",
		"Array of ADC group IDs" ) );
      Desc( SE( "positionADC", "INT_8U[nADC][nFrame]",
		"Array of lists of FrAdcData offset positions, in bytes,"
		" from beginning of file (size of nFrame*nADC)"
		) );
    }

    inline const FrTOCAdcData::MapADC_type& FrTOCAdcData::
    GetADC( ) const
    {
      return m_info;
    }

  } // namespace - Version_8
} // namespace - FrameCPP

#endif /* FrameCPP_VERSION_8_FrTOCAdcData_HH */
